
CREATE OR REPLACE LIBRARY mail_utl_l
AS
   '/oracle/local/lib/libmail_utl.so';

CREATE OR REPLACE 
PACKAGE mail_utl
IS
    FUNCTION send_mail
       (return_address  IN VARCHAR2 DEFAULT NULL,
        recipient_list  IN VARCHAR2,
        subject_line    IN VARCHAR2 DEFAULT NULL,
        message_text    IN VARCHAR2 DEFAULT NULL)
    RETURN PLS_INTEGER;
    PRAGMA RESTRICT_REFERENCES (send_mail, WNDS, RNDS, WNPS, RNPS);

END mail_utl;
/

CREATE OR REPLACE
PACKAGE BODY mail_utl
IS
    FUNCTION send_mail
       (return_address  IN VARCHAR2,
        recipient_list  IN VARCHAR2,
        subject_line    IN VARCHAR2,
        message_text    IN VARCHAR2)
    RETURN PLS_INTEGER
    IS EXTERNAL
       LIBRARY mail_utl_l
       NAME "send_mail"
       LANGUAGE C
       WITH CONTEXT     -- passes context so we can raise exception
       PARAMETERS
          (CONTEXT,     -- keyword giving location of CONTEXT parameter
           return_address STRING,
           return_address INDICATOR,
           recipient_list STRING,
           recipient_list INDICATOR,
           subject_line   STRING  ,
           subject_line   INDICATOR,
           message_text   STRING,
           message_text   INDICATOR,
           RETURN sb4);
END mail_utl;
/


SET SERVEROUTPUT ON SIZE 1000000

DECLARE
   exit_status  PLS_INTEGER;
BEGIN
   /* This call will succeed and "fill-in" the other NULL parameters */
   exit_status := mail_utl.send_mail (recipient_list => 'fred');
   DBMS_OUTPUT.PUT_LINE ('send_mail exit_status = ' || exit_status);


   /* This call will fail since the recipient list is NULL.
   || Our unnamed programmer-defined exception will be raised as a result.
   */
   exit_status := mail_utl.send_mail (recipient_list => NULL,
                                      subject_line   => 'Mail from PL/SQL!');
   DBMS_OUTPUT.PUT_LINE ('send_mail exit_status = ' || exit_status);
END;
/
