
CREATE OR REPLACE LIBRARY rand48_utl_l
AS
   '/oracle/local/lib/librand48_utl.so';

CREATE OR REPLACE 
PACKAGE rand48_utl
IS
    FUNCTION nrand48
       (state0 IN OUT PLS_INTEGER,
        state1 IN OUT PLS_INTEGER,
        state2 IN OUT PLS_INTEGER )
    RETURN PLS_INTEGER;
    PRAGMA RESTRICT_REFERENCES (nrand48, WNDS, RNDS, WNPS, RNPS);

END rand48_utl;
/

CREATE OR REPLACE
PACKAGE BODY rand48_utl
IS
    FUNCTION nrand48
       (state0 IN OUT PLS_INTEGER,
        state1 IN OUT PLS_INTEGER,
        state2 IN OUT PLS_INTEGER)
    RETURN PLS_INTEGER
    IS EXTERNAL
       LIBRARY rand48_utl_l
       NAME "plsql_nrand48"
       LANGUAGE C
       PARAMETERS
          (state0 ub2,
           state1 ub2,
           state2 ub2,
           RETURN sb4);
END rand48_utl;
/


-- Now call the external function a few times

SET SERVEROUTPUT ON SIZE 1000000

DECLARE
   state0      PLS_INTEGER;
   state1      PLS_INTEGER;
   state2      PLS_INTEGER;
   rnd_value   PLS_INTEGER;
BEGIN
   /* For the first loop, use constant "seed" state values (all zeros)
   || so that the same sequence of pseudo-random values are returned
   || each time this loop is executed.
   */ 
   state0 := 0;
   state1 := 0;
   state2 := 0;

   DBMS_OUTPUT.PUT_LINE ( 'state = ' ||
                           state0 || ' ' || state1 || ' ' || state2 );

   FOR v_cnt IN 1 .. 10 LOOP
      rnd_value := rand48_utl.nrand48 (state0, state1, state2);
      DBMS_OUTPUT.PUT_LINE ('static nrand48() call #' || v_cnt || 
                            ' returns ' || rnd_value);
   END LOOP;

   /* Next, reset the state values with arbitrary dynamic data.
   || The resulting sequence of pseudo-random values should be different 
   || each time this example is executed.
   */
   state0 := BITAND(TO_CHAR(SYSDATE, 'SSSSS'),    65534);
   state1 := BITAND(TO_CHAR(SYSDATE, 'SSHH24MI'), 65534);
   state2 := BITAND(TO_CHAR(SYSDATE, 'JSS'),      65534);

   DBMS_OUTPUT.PUT_LINE ( 'state = ' ||
                           state0 || ' ' || state1 || ' ' || state2 );

   FOR v_cnt IN 1 .. 10 LOOP
      rnd_value := rand48_utl.nrand48 (state0, state1, state2);
      DBMS_OUTPUT.PUT_LINE ('dynamic nrand48() call #' || v_cnt || 
                            ' returns ' || rnd_value);
   END LOOP;
END;
/