
CREATE OR REPLACE LIBRARY libc_1
AS
   '/lib/libc.so';

CREATE OR REPLACE 
PACKAGE random_utl
IS
    FUNCTION rand
    RETURN PLS_INTEGER;
    PRAGMA RESTRICT_REFERENCES (rand, WNDS, RNDS, WNPS, RNPS);

    PROCEDURE srand
       (seed IN PLS_INTEGER);
    PRAGMA RESTRICT_REFERENCES (srand, WNDS, RNDS, WNPS, RNPS);

END random_utl;
/

CREATE OR REPLACE
PACKAGE BODY random_utl
IS
    /* Tested with: (1) Solaris 2.5.1 and Oracle 8.0.3
    ||              (2) Windows NT 4.0 and Oracle 8.0.3
    */
    FUNCTION rand RETURN PLS_INTEGER
    /* Return a random number between 1 and (2**16 -1), using
    || the current seed value
    */
    IS EXTERNAL
       LIBRARY libc_1
       NAME "rand"
       LANGUAGE C;

    PROCEDURE srand (seed IN PLS_INTEGER)
    /* Store a seed value used by external rand() function */
    IS EXTERNAL
       LIBRARY libc_1
       NAME "srand"
       LANGUAGE C
       PARAMETERS (seed ub4);
END random_utl;
/


-- Now call the external function a few times

SET SERVEROUTPUT ON SIZE 1000000

DECLARE
   rnd_value   PLS_INTEGER;
   seed        PLS_INTEGER;
BEGIN
   /* Generate a seed value from the current system time. */
   SELECT TO_CHAR(SYSDATE, 'SSSSS') INTO seed FROM DUAL;

   /* Call the srand external procedure to store our seed in memory. */
   random_utl.srand (seed);

   /* Now demonstrate some random numbers. */
   FOR v_cnt IN 1 .. 10 LOOP
      rnd_value := random_utl.rand;
      DBMS_OUTPUT.PUT_LINE ('rand() call #' || v_cnt || 
                            ' returns ' || rnd_value);
   END LOOP;
END;
/